@@ -3,6 +3,7 @@ package Text::Editor::Vip::Buffer;
 
 use strict;
 use warnings ;
+use Data::TreeDumper ;
 
 BEGIN 
 {
@@ -18,6 +19,7 @@ $VERSION     = 0.01;
 
 #-------------------------------------------------------------------------------
 
+use Time::HiRes ;
 use Carp qw(carp confess cluck);
 use List::Util qw(min) ;
 
@@ -46,7 +48,7 @@ selection,  undo and plugins.
 
 =cut
 
-my $uid = 0 ;
+my $uid = 0 ; #well, hmm, good enough
 
 sub new
 {
@@ -98,10 +100,13 @@ Helper sub called by new. This is considerer private.
 =cut
 
 my $buffer = shift ;
+my $expansions = $buffer->{EXPANSIONS} || [] ;
 
 %$buffer = 
 	(
 	  NODES                        => new Text::Editor::Vip::Buffer::List()
+	, EXPANSIONS                   => $expansions
+	
 	, MARKED_AS_EDITED             => 0
 	
 	, DO_PREFIX                    => ''
@@ -110,6 +115,7 @@ my $buffer = shift ;
 	, UNDO_STACK                   => []
 	, REDO_STACK                   => []
 	
+	, CLIPBOARDS                   => {}
 	, MODIFICATION_LINE            => 0
 	, MODIFICATION_CHARACTER       => 0
 	, SELECTION                    => new Text::Editor::Vip::Selection()
@@ -140,7 +146,7 @@ sub ExpandedWithOrLoad
 
 =head2 ExpandedWithOrLoad
 
-See L<PLUGGINS>.
+See L<PLUGINS>.
 
 If the name passed as first argument doesn't match a sub within the object, the module, passed as second argument,
 is loaded.
@@ -173,11 +179,14 @@ eval "\$sub = ${class}->can('${sub_name}') ;" ;
 
 if(defined $sub)
 	{
+	push @{$buffer->{EXPANSIONS}}, {CALLER => [caller()],  LOOKING_FOR => $sub_name, FOUND => 1} ;
 	return(1) ;
 	}
 else
 	{
-	$buffer->LoadAndExpandWith($module) ;
+	$buffer->LoadAndExpandWith($module, 1) ;
+	
+	push @{$buffer->{EXPANSIONS}}, {CALLER => [caller()],  LOOKING_FOR => $sub_name, LOADING_MODULE => $module} ;
 	return(0) ;
 	}
 }
@@ -189,7 +198,7 @@ sub LoadAndExpandWith
 
 =head2 LoadAndExpandWith
 
-See L<PLUGGINS>.
+See L<PLUGINS>.
 
 Loads a perl module (plugin) and adds all it functionality to the buffer
 
@@ -202,8 +211,9 @@ Loads a perl module (plugin) and adds all it functionality to the buffer
 
 # look at Export::Cluster, Export::Dispatch
 
-my $buffer =  shift ;
-my $module = shift ;
+my $buffer     = shift ;
+my $module     = shift ;
+my $no_history = shift ;
 
 eval "use $module ;" ;
 die __PACKAGE__ . " couldn't load '$module':\n$@\n" if $@ ;
@@ -217,6 +227,11 @@ $buffer->PushUndoStep
 		, "# undo for \$buffer->LoadAndExpandWith('$module') ;"
 		) ;
 
+unless($no_history)
+	{
+	push @{$buffer->{EXPANSIONS}}, {CALLER => [caller()],  LOADING_MODULE => $module} ;
+	}
+
 #alternative way to expend the object
 
 # expands the current's package ISA not the objects isa
@@ -247,7 +262,7 @@ sub ExpandWith
 
 =head2 ExpandWith
 
-See L<PLUGGINS>.
+See L<PLUGINS>.
 
 Adds a sub to a buffer instance. 
 
@@ -286,23 +301,59 @@ my $class = ref($buffer) ;
 my $warning = '' ;
 local $SIG{'__WARN__'} = sub {$warning = $_[0] ;} ;
 
+#~ $DB::single = 1 ;
+
+my ($package, $file_name, $line) = caller() ;
+$package ||= '' ;
+
+my $location = "$file_name:$line" ;
+
 if($sub)
 	{
+	die __PACKAGE__ . " not a sub reference '$sub_name' at $location:\n" unless 'CODE' eq ref $sub ;
+	
 	eval "*${class}::${sub_name} = \$sub;" ;
+	push @{$buffer->{EXPANSIONS}}, {CALLER => [caller()],  SUB_REF => $sub_name} ;
 	}
 else
 	{
 	# load the named sub from the caller package
 	
-	my ($package, $file_name, $line) = caller() ;
-	$package ||= '' ;
-	
+	die __PACKAGE__ . " error exapnding with undefined named sub at $location\n" unless defined $sub_name and $sub_name ne '' ;
+
+	my $found_sub ;
+	eval "\$found_sub = ${package}->can('${sub_name}') ;" ;
+	die __PACKAGE__ . " error exapnding with named sub '$sub_name' at $location.\n" unless defined $found_sub ;
+
 	eval "*${class}::${sub_name} = \\\&$package\::${sub_name};" ;
+	die __PACKAGE__ . " error exapnding with named sub ''$sub_name' at $location:\n$@\n" if $@ ;
+
+	push @{$buffer->{EXPANSIONS}}, {CALLER => [$package, $file_name, $line],  SUB_REF_IN_CALLER_SPACE => $sub_name} ;
 	}
 }
 
 #-------------------------------------------------------------------------------
 
+sub PrintExpansionHistory
+{
+
+=head2 PrintExpansionHistory
+
+Displays the expansion done to the buffer
+
+=cut
+
+my $buffer = shift ;
+my $message = shift || '' ;
+
+my ($package, $file_name, $line) = caller() ;
+$message .= " @ '$file_name:$line'" ;
+
+print DumpTree($buffer->{EXPANSIONS}, $message) ;
+}
+
+#-------------------------------------------------------------------------------
+
 sub Do
 {
 
@@ -439,7 +490,7 @@ sub GetLineAttribute
 Retrieves  a named attribute from a line. 
 
   $buffer->SetLineAttribute(0, 'TEST', $some_data) ;
-  $retrieved_data = $buffer->GetLineAttribute(0'TEST', $some_data) ;
+  $retrieved_data = $buffer->GetLineAttribute(0, 'TEST') ;
 
 =cut
 
@@ -465,12 +516,18 @@ Used to mak the buffer as unedited You should not need to use this function.
 
 Used to query the buffer about its state. Returns (1) if the buffer was edit. (0) otherwise.
 
+=head2 GetLastEditionTImestamp
+
+Returns the time of the last edition.
+
 =cut
 
 sub IsBufferMarkedAsEdited {return($_[0]->{MARKED_AS_EDITED}) ;}
-sub MarkBufferAsEdited { $_[0]->{MARKED_AS_EDITED} = 1 ;}
+sub MarkBufferAsEdited { $_[0]->{MARKED_AS_EDITED} = 1 ; $_[0]->{EDITED_AT_TIME} = Time::HiRes::time() ;}
 sub MarkBufferAsUnedited {$_[0]->{MARKED_AS_EDITED} = 0 ;}
 
+sub GetLastEditionTImestamp {$_[0]->{EDITED_AT_TIME};}
+
 #-------------------------------------------------------------------------------
 
 sub GetNumberOfLines
@@ -487,6 +544,20 @@ return($_[0]->{NODES}->GetNumberOfNodes()) ;
 
 #------------------------------------------------------------------------------
 
+sub GetLastLineIndex
+{
+
+=head2 GetLastfLineIndex
+
+Returns theindex of the last line. the buffer always contains at least one line thus the last line index is always 0 or more.
+
+=cut
+
+return($_[0]->{NODES}->GetNumberOfNodes() - 1) ;
+}
+
+#------------------------------------------------------------------------------
+
 sub GetModificationPosition
 {
 
@@ -525,11 +596,11 @@ $buffer->SetModificationCharacter($character) ;
 sub OffsetModificationPosition
 {
 
-=head2 SetModificationPosition
+=head2 OffsetModificationPosition
 
-Sets the position, line and character, where the next modification will occure.
+Offset the position, line and character, where the next modification will occure. an exception is thrown if position is not valid
 
-   $buffer->SetModificationPosition(0, 15) ;
+   $buffer->OffsetModificationPosition(0, 15) ;
 
 =cut
 
@@ -550,6 +621,40 @@ $buffer->SetModificationCharacter($buffer->GetModificationCharacter() + $charact
 
 #-------------------------------------------------------------------------------
 
+sub OffsetModificationPositionGuarded
+{
+
+=head2 OffsetModificationPositionGuarded
+
+Offsets the position, line and character, where the next modification will occure. Nothing happends if the new position is invalid
+
+   $buffer->OffsetModificationPositionGuarded(0, 15) ;
+
+=cut
+
+my ($buffer, $line_offset, $character_offset) = @_ ;
+
+my $new_line = $buffer->GetModificationLine() + $line_offset ;
+my $new_character = $buffer->GetModificationCharacter() + $character_offset ;
+
+if
+	(
+ 	   $new_line < $buffer->GetNumberOfLines()
+	&& 0 <= $new_line
+	&& 0 <= $new_character
+	)
+	{
+	$buffer->OffsetModificationPosition($line_offset, $character_offset) ;
+	return(1) ;
+	}
+else
+	{
+	return(0) ;
+	}
+}
+
+#-------------------------------------------------------------------------------
+
 sub GetModificationLine
 {
 
@@ -666,10 +771,10 @@ See L<GetLineText>.
 
 =cut
 
-my $buffer         = shift ;
+my $buffer       = shift ;
 my $a_line_index = shift ;
 
-return($buffer->{NODES}->GetNodeData($a_line_index)) ;
+return( $buffer->{NODES}->GetNodeData($a_line_index) ) ;
 }
 
 #-------------------------------------------------------------------------------
@@ -686,7 +791,7 @@ Returns the text of the line passes as argument or the current modification line
 
 =cut
 
-my $buffer = shift ;
+my $buffer       = shift ;
 my $a_line_index = shift ;
 
 $a_line_index = $buffer->GetModificationLine() unless defined $a_line_index ;
@@ -704,6 +809,44 @@ else
 
 #-------------------------------------------------------------------------------
 
+sub GetLineTextWithNewline
+{
+
+=head2 GetLineTextWithNewline
+
+Returns the text of the line passes as argument or the current modification line if no argument is passed. A "\n" is
+appended if the line is not the last line in the buffer.
+
+  my $line_12_text = $buffer->GetLineTextWithNewline(12) ;
+  my $current_line_text = $buffer->GetLineTextWithNewline() ;
+
+=cut
+
+my $buffer       = shift ;
+my $a_line_index = shift ;
+
+$a_line_index = $buffer->GetModificationLine() unless defined $a_line_index ;
+
+if(0 <= $a_line_index && $a_line_index < $buffer->GetNumberOfLines())
+	{
+	if($a_line_index == $buffer->GetLastLineIndex())
+		{
+		return($buffer->GetLine($a_line_index)->{TEXT}) ;
+		}
+	else
+		{
+		return($buffer->GetLine($a_line_index)->{TEXT} . "\n") ;
+		}
+	}
+else
+	{
+	$buffer->PrintError("GetLineText: Invalid line index: $a_line_index. Number of lines: " . $buffer->GetNumberOfLines(). "\n") ;
+	return('') ;
+	}
+}
+
+#-------------------------------------------------------------------------------
+
 sub GetLineLength
 {
 
@@ -1097,10 +1240,11 @@ NO_SMART_INDENTATION is defined in Text::Editor::Vip::Buffer::Constants.
 
 =cut
 
-my $buffer                  = shift ;
-my $text_to_insert        = shift || '' ;
+my $buffer                = shift ;
+my $text_to_insert        = shift ;
 my $use_smart_indentation = shift || SMART_INDENTATION ;
 
+$text_to_insert ='' unless defined $text_to_insert ;
 my @text_to_insert ;
 
 if(ref($text_to_insert) eq 'ARRAY')
@@ -1171,7 +1315,7 @@ for(@text_to_insert)
 
 =head1 PLUGINS
 
-Vip::Buffer has a very simple pluggin system. You can add a function to  the buffer with
+Vip::Buffer has a very simple plugin system. You can add a function to  the buffer with
 L<ExpandWith>, L<LoadAndExpandWith> and  L<ExpandedWithOrLoad>. The functions
 added through plugins are made available to the instance, calling the plugin sub, only. 
 
